package PureMessage::Policy::Virus::Sophos;

use strict;

our $VERSION = '5.500';

use base 'PureMessage::Policy::Virus';
use Sophos::SAVI::Engine;
use PureMessage::Logger;

sub get_engine {
    my $my_config = shift;

    my ($datadir, %opts);
    while (my ($k, $v) = each %$my_config) {
	next if $k eq 'enabled' or $k eq 'daemon';
	if ($k eq 'datadir') {
	    $datadir = $v;
	    next;
	}
	$opts{$k} = $v if defined $v;
    }
    $opts{logger} = PureMessage::Logger->new(ident=>"Sophos-Engine");
    my %RANK = (
	GrpSuper		=> 2,
	GrpArchiveUnpack	=> 1,
	GrpSelfExtract		=> 1,
	GrpExecutable		=> 1,
	GrpInternet		=> 1,
	GrpMSOffice		=> 1,
	GrpMisc			=> 1,
	GrpDisinfect		=> 1,
	GrpClean		=> 1,
    );
    my @opts = map { $_ => $opts{$_} }
	       sort { ($RANK{$b}||0) <=> ($RANK{$a}||0) || ($a cmp $b) }
	       keys %opts;

    # Create the engine here, and let the closures pick it up.
    Sophos::SAVI::Engine->new($datadir, @opts);
}

sub impl {
    my ($pkg, $my_config) = @_;
    my $engine = get_engine($my_config);

    [
	sub {
	    my ($self, $msg, $filename) = @_;
	    my @viruses;
	    my $report = $engine->scan_file($filename);

            # Report hash is list of viruses found for each (part) filename -
            # we only want the complete list
            return @viruses = map { @$_ } values %$report;
	},
	# Clean: must return one of qw(novirus cleanfailed cleaned deleted)
        # We can't try to clean the full message, because we can't reconstruct
        # the headers after we've cleaned the body
	sub {
            my ($self, $msg, $filename) = @_;
            my $report = $engine->clean_file($filename);
            my $status = $report->{$filename};
	},
    ]
}

sub daemon {
    my ($pkg, $my_config) = @_;
    my $engine = get_engine($my_config);

    [
	sub {
	    my ($self, $filename) = @_;
	    my @viruses;
	    my $report = $engine->scan_file($filename);

            # Report hash is list of viruses found for each (part) filename -
            # we only want the complete list
            return @viruses = map { @$_ } values %$report;
	},
	# Clean: must return one of qw(novirus cleanfailed cleaned deleted)
	sub {
	    my ($self, $filename) = @_;
	    my $report = $engine->clean_file($filename);
	    my $status = $report->{$filename};
	    return $status;
	},
    ]
}

1;
